part of ipa;

class IPA {

  HttpRouter _router;

  /**
   * Initialize an IPA instance
   *
   * @param HttpRouter this._router - The router to use
   */
  IPA(this._router) {
    this._router.serviceManager.add(
      "__IPA_GLOBAL_MIDDLEWARE",
      new IPAGlobalMiddlewareFactory()
    );
  }

  /**
   * Integrates an IPA global middleware
   *
   * @param AbstractMiddleware middleware - The middleware to plug in
   * @return IPA                          - This IPA instance
   */
  IPA use(AbstractMiddleware middleware) {
    var promise = this._router.serviceManager.get("__IPA_GLOBAL_MIDDLEWARE");
    promise.then((middlewares) {
      middlewares.add(middleware);
    });
    return this;
  }

  /**
   * Bind this IPA instance to a port and start accepting requests
   *
   * @param String host - The host to bind to
   * @param int port    - The port to listen on
   * @return IPA        - This IPA instance
   */
  IPA listen(String host, int port) {
    HttpServer.bind(host, port).then((HttpServer server) {
      server.pipe(this._router);
    });
    return this;
  }

}